#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <signal.h>
#include "vector2.H"
#include "symplex.H"
#include "symplex.C"

// #define DEBUG
#define MYNAME "divano"
#define VERSION "1.0"


typedef double Real;

int interrupt=0;
void ClearInterrupt(void) {
  if (interrupt)
    cout<<"Signal catched, exiting loop\n";
  interrupt=0;
}


double fitt=0.0;
double levelstart=0,levelstep=0.5;
int levelcurves=1;
int csiarrows=1;

char *file="out.ps";
char *savefile="symplex.save";

class point: public vector2 {
public:
  vector2 csi;
  vector2 dato;
  vector2 oldcsi;
  double dist; /*divcsi in questo caso*/
  point(vector2 x,vector2 val,double val2):vector2(x){csi=val;dato=vector2(0,0);dist=val2;};
  point(vector2 x):vector2(x){csi=vector2(0,0);dato=vector2(0,0);};
  point():vector2(0,0){csi=vector2(0,0);dato=vector2(0,0);};
  int bordo;
  int mark;
};

typedef Complex<point> Cpx;
typedef Symplex<point> Spx;
typedef Vertex<point> Vtx;

ostream& operator<<(ostream &out,point &p) {
  return out<<p.x[0]<<" "<<p.x[1]<<" "<<p.csi.x[0]<<" "<<p.csi.x[1];
}

istream& operator>>(istream &in,point &p) {
  return in>>p.x[0]>>p.x[1]>>p.csi.x[0]>>p.csi.x[1];
}

point Media(point &p1,point &p2) {
  return point((p1+p2)/2.0,(p1.csi+p2.csi)/2.0,(p1.dist+p2.dist)/2.0);
}

double Dist(point &p1,point &p2) {
  double r;
  r=abs(p1-p2)*(1.0+fitt*fabs(p1.dist-p2.dist));
  //r=abs(p1-p2)*(abs(p1+p2)/2.0);
  return r;
}

double PlaneDist(point &p1,point &p2) {
  return abs(p1-p2);
}


//#include "symplex.C"

//typedef vector2 point; //tipo di base per i vertici



void PsBegin(ostream &out)
{
  out<<"%!PS-Adobe-2.0 EPSF-2.0\n"
        <<"%%Title: Spx Output\n"
        <<"%%Creator: symplex.C manu-fatto\n"
    //        <<"%%Pages: 1\n"
    //    <<"%%DocumentFonts:\n"
        <<"%%EndComments\n";
  out <<"/newpage {";
  out <<"250 250 scale 1.1 1.4  translate 0.002 setlinewidth\n";
  out <<"/np {newpath} def /mt {moveto} def /lt {lineto stroke} def\n";
  out <<"/Times-Roman findfont 0.08 scalefont setfont\n";
  out <<" }def\n\n";
}

void PsEnd(ostream &out)
{
  //  out <<"showpage\n";
}

void PsWrite(ostream &out,const vector2 &p)
{
  out<<p.x[0]<<" "<<p.x[1]<<" ";
}

void PsWrite(ostream &out,Cpx *c) {
  for (Cpx::iterator i=c->begin();i!=c->end();++i)
    {
      if ((*i)->Dim()==1) //e' un segmento
	{
	  out<<"np ";
	  PsWrite(out,(*i)->Point(0));
	  out<<" mt ";
	  PsWrite(out,(*i)->Point(1));
	  out<<" lt\n";
	}
    }
}

void PsFile(Cpx *c,char *filename) {
  ofstream out(filename);
  PsBegin(out);
  PsWrite(out,c);
  PsEnd(out);
  cout<<"Triangulation saved on file "<<filename<<"\n";
}

double Area(Spx *s) {
  //*s dev'essere un triangolo, torna l'area
#ifdef DEBUG
  if (s->Dim()!=2) Error("Area(): triangle expected");
#endif DEBUG
  double a=0;
  point *p[3];
  for (int i=0;i<3;i++)
    p[i]=&s->Point(i);
  for (int i=0;i<3;i++)
    a+=(p[(i+1)%3]->x[1]+p[i]->x[1])*(p[i]->x[0]-p[(i+1)%3]->x[0])/2.0;
  if (a<=0) Error("negative area!");
  return a;
}

double Area(Cpx *c) {
  double tot=0;
  for (int i=0;i<c->N(2);i++) 
    tot+=Area(c->Find(2,i));
  return tot;
}
void Aggiusta(Cpx *c) {
  Spx *lato;
  int I[2];
  vector <Vtx *>V(2);
  vector <Vtx *>t1(3);
  vector <Vtx *>t2(3);
  Spx *T[2];
  int n;
  int fatti=0;
  vector2 alpha,beta,gamma;
  double s;
  n=c->N(1);
  for (int i=0;fatti<n;i++) {
    i%=n;
    lato=c->Find(1,i);
    fatti++;
    if (lato->Conta(2)==2) {
      for (int j=0;j<2;j++) {
	T[j]=lato->Find(2,j);
	int k;
	for (k=0;k<3 && lato->Contains(T[j]->Vert(k));k++);
	I[j]=k;
	V[j]=T[j]->Vert(k);
      }
      alpha=lato->Point(1)-lato->Point(0);
      beta=V[1]->Point()-V[0]->Point();
      gamma=V[0]->Point()-lato->Point(0);
      s=(beta.x[1]*gamma.x[0]-beta.x[0]*gamma.x[1])/(alpha.x[0]*beta.x[1]-beta.x[0]*alpha.x[1]);
      
      if (s>0 && s<1 && PlaneDist(V[0]->Point(),V[1]->Point())/PlaneDist(lato->Point(0),lato->Point(1))<0.98) {
	//scambia
	Spx *newlato;
	t1[0]=V[0];
	t1[1]=T[0]->Vert((I[0]+1)%3);
	t1[2]=V[1];
	t2[0]=V[0];
	t2[1]=V[1];
	t2[2]=T[0]->Vert((I[0]+2)%3);
	newlato=c->NewSymplex(V,2);
	c->NewSymplex(t1,3);
	c->NewSymplex(t2,3);
	
	c->Delete(T[0]);
	c->Delete(T[1]);
	c->Delete(lato);
	fatti=0;
      }
    }
  }
}

void Triangola(Cpx *c,double eps) {
  double max;
  double f;
  Spx *found;
  while (!interrupt) {
    max=0;
    for(Cpx::iterator i=c->Begin(1);i!=c->end() && i!=c->End(2);i++) {
      f=Dist(((*i)->Point(0)),((*i)->Point(1)));
      if (f>max) {
	max=f;
	found=*i;
      }
    }
    if (max>eps) {
      c->Triangola(found,c->NewSymplex(Media(found->Point(0),found->Point(1))));
      Aggiusta(c);
    }
    else return;
  }
  ClearInterrupt();
}

void Triangola2(Cpx *c,double eps) {
  double max;
  double f;
  int end;
  Spx *found;
  end=0;
  for(Cpx::iterator i=c->begin();;i++) {
    if (i>=c->end()) {
      i=c->begin(); 
      if (end==1) return;
      end=1;
    };
    if ((*i)->Dim()==1) {
      f=abs(((*i)->Point(0))-((*i)->Point(1)));
      if (f>eps) {
	c->Triangola(*i,c->NewSymplex(Media((*i)->Point(0),(*i)->Point(1))));
	i=c->begin();
	end=0;
      }
    }
  }
}



Cpx *poligono(int n) {
  Cpx *res=new Cpx;
  vector<Vtx*> v(n);
  vector<Vtx*> t(3);
  Vtx *c;
  
  res->Check();
  
  c=res->NewSymplex(point(vector2(0,0)));

  res->Check();

  for (int i=0;i<n;i++) {
    v[i]=res->NewSymplex(vector2(cos((2.0*i+1)*M_PI/n),sin((2.0*i+1)*M_PI/n)));
    res->Check();
      
  }
  for (int i=0;i<n;i++) {
    t[0]=c;
    t[1]=v[i];
    t[2]=v[(i+1)%n];
    res->NewSymplex(t);
    res->Check();
    
  }
  res->Check();
  return res;
}

Cpx *esagono(double s=0.25,int n=6) {
  Cpx *res=new Cpx;
  vector<Vtx*> v(n);
  vector<Vtx*> t(3);
  Vtx *c;
  
  res->Check();
  
  c=res->NewSymplex(point(vector2(0,0)));

  res->Check();

  double alpha=(M_PI*(1.0+2.0/n))/2.0;
  double theta,sgn;

  for (int i=0;i<n;i++) {    
    theta=(2.0*i+1.0)*M_PI/n;
    alpha=-alpha;
    v[i]=res->NewSymplex(vector2(cos(theta),sin(theta))+vector2(s*cos(theta+alpha),s*sin(theta+alpha)));
    res->Check();
      
  }
  for (int i=0;i<n;i++) {
    t[0]=c;
    t[1]=v[i];
    t[2]=v[(i+1)%n];
    res->NewSymplex(t);
    res->Check();
    
  }
  res->Check();
  return res;
}

Cpx *doppiodivano(double a=1,double b=1,double c=2,double d=0.1) {
  Cpx *res=new Cpx;
  vector<Vtx*> v(8);
  vector<Vtx*> t(3);
  Vtx *x;
  
  v[0]=res->NewSymplex(vector2(0,d));
  v[1]=res->NewSymplex(vector2(0,0.75));
  v[2]=res->NewSymplex(vector2(-a,0.75));
  v[3]=res->NewSymplex(vector2(-a,-0.75));
  v[4]=res->NewSymplex(vector2(0,-0.75));
  v[5]=res->NewSymplex(vector2(0,-d));
  v[6]=res->NewSymplex(vector2(c-a,-d));
  v[7]=res->NewSymplex(vector2(c-a,d));
  x=res->NewSymplex(vector2(-0.5,0));

  for (int i=0;i<8;i++) {
    t[0]=x;
    t[1]=v[i];
    t[2]=v[(i+1)%8];
    res->NewSymplex(t);
  }

  return res;  
}


Cpx *divano(double a=1,double b=1,double c=2,double d=0.25) {
  Cpx *res=new Cpx;
  vector<Vtx*> v(8);
  vector<Vtx*> t(3);
  Vtx *x;
  
  v[0]=res->NewSymplex(vector2(0,0));
  v[1]=res->NewSymplex(vector2(0,b-d));
  v[2]=res->NewSymplex(vector2(-a,b-d));
  v[3]=res->NewSymplex(vector2(-a,-d));
  v[4]=res->NewSymplex(vector2(0,-d));
  v[5]=res->NewSymplex(vector2(c-a,-d));
  v[6]=res->NewSymplex(vector2(c-a,0));
  x=v[0];

  for (int i=1;i<6;i++) {
    t[0]=x;
    t[1]=v[i];
    t[2]=v[i+1];
    res->NewSymplex(t);
  }

  return res;  
}


class matricetta {
public:
  vector2 v[2]; //le righe della matricetta
  vector2 operator*(vector2 &w) {return vector2(v[0]*w,v[1]*w);}
};

int Index(Spx *T,Spx *V) {
  Spx::iterator i;
  for (i=T->Pos(0);i!=T->end() && (*i)!=V;i++);
#ifdef DEBUG
  if (i==T->end()) Error("Index not found");
#endif
  return i-T->Pos(0);
}

/***********************************************************
************************************************************
***********************************************************/


double max(double x,double y) {
  return x>y? x:y;
}

double sqr(double x) {
  return x*x;
}

/*
double phi(const vector2 v) {//Wulff shape
  return max(fabs(v.x[0]),fabs(v.x[1]));
}
*/

static int nWulff=2; /*numero di lati/2*/

double phi(const vector2 v) {
//Wulff shape 2nWulff-agonale circoscritta al cerchio unitario
  static double m,x;
  m=0;
  for (int i=0;i<nWulff;i++) {
    x=fabs(v*vector2(cos((M_PI/nWulff)*i),sin((M_PI/nWulff)*i)));
    if (x>m) m=x;
  }
  return m;
}


/*
void WulffProj(vector2 &v) {
  for (int i=0;i<2;i++)
    if (fabs(v.x[i])>1.0) v.x[i]/=fabs(v.x[i]);
}
*/

void WulffProj(vector2 &v) {
  if (phi(v)>1)
    v/=phi(v);
} 


void BordoProj(point &x) {
  if (abs(x.dato)<1.01) {
    x.csi -= (x.csi*x.dato)*x.dato-x.dato;
    WulffProj(x.csi);
  }
  else
    x.csi=x.dato;
}

vector<matricetta> M; //massa
vector<vector2> grads[3]; //gradientini
// vector<int> vert[3]; //indici dei vertici dei triangoli
// vector<int> bordo; //il vertice e' di bordo?
// vector<vector2 > dato; //dato iniziale
vector<double> divcsi; //divergenza di csi sui triangoli

int nV,nT;


void CalcolaDist(Cpx *c) {
  Vtx *x;
  double f;
  double a,A;
  for (int i=0;i<nV;i++) {
    x=c->Vert(i);
    f=0;
    A=0;
    for (Spx::iterator j=x->Pos(2);j!=x->end() && (*j)->Dim()<=2;j++) {
      if ((*j)->Dim()==2) {
	a=Area(*j);
	f+=a*divcsi[c->IndexFind(*j)];
	A+=a;
      }
    }
    x->Point().dist=f/A;
    // x->Point().dist=abs(x->Point());
  }
}

double Energia(Cpx *c) {
  double e=0,f;
  double A;
  Spx *T;
  for (int i=0;i<nT;i++) {
    A=Area(T=c->Find(2,i));
    f=0;
    for (int j=0;j<3;j++) {
      f+=T->Point(j).csi * (((grads[j]))[i]);
    }
    (divcsi)[i]=f;
    e+=A*sqr(f);    
  }
  return e;
}

void PsWritecsi(ostream &out, Cpx *c) {
  out<<"/arrow {0.2 mul /dy exch def 0.2 mul /dx exch def /y exch def /x exch def\n"
    "newpath x y moveto dx dy rlineto stroke} def\n";
  for (int i=0;i<c->N(0);i++) {
    PsWrite(out,(vector2)(c->Point(i)));
    out<<" ";
    PsWrite(out,c->Point(i).csi);
    out<<" arrow\n";
  }  
}

void PsWritedivcsi(ostream &out, Cpx *c) {
  vector2 v;
  Spx *T;
  double min,max;
  double dist[3];
  double lambda[3];
  min=max=divcsi[0];
  for (int i=0;i<nT;i++) {
    if (divcsi[i]>max) max=divcsi[i];
    else if (divcsi[i]<min) min=divcsi[i];    
  }
  if (max<=min) max=min+0.001;
  out<<"gsave\n";
  out<<"/sg {"<<min<<" sub "<<max-min<<" div 0.8 mul 0.1 add 0 exch 1 setrgbcolor} def\n";
  out<<"/Times-Roman findfont 0.04 scalefont setfont\n";
  out<<"/sh {show} def\n";
  if (levelcurves) CalcolaDist(c);

  for (int i=0;i<nT;i++) {
    T=c->Find(2,i);
    if (!levelcurves) {
      out<<(divcsi)[i]<<" sg\n";
      out<<"np ";
      PsWrite(out,(vector2) T->Point(0));
      out<<"mt ";
      PsWrite(out,(vector2) T->Point(1));
      out<<"lineto ";
      PsWrite(out,(vector2) T->Point(2));
      out<<"lineto closepath fill\n";
    } else {
      double min,max;
      //begin level curves
      for (int j=0;j<3;j++) {
	dist[j]=T->Point(j).dist;
	if (j==0 || dist[j]<min) min=dist[j];
	if (j==0 || dist[j]>max) max=dist[j];
      }
      for (double t=levelstart+levelstep*(int)((min-levelstart)/levelstep);
	   t<=max;t+=levelstep) {
	for (int j=0;j<3;j++) {
	  if (dist[j]==dist[(j+1)%3]) lambda[j]=0;
	  else lambda[j]=(t-dist[(j+1)%3])/(dist[j]-dist[(j+1)%3]);
	}
	out<<t<<" sg\n";
	out<<"np ";
	int n=0;
	vector2 x;
	Spx::iterator l=NULL;
	for (int j=0;j<3 && n<2;j++) {
	  if (lambda[j]>=0.0 && lambda[j]<=1.0) {
	    PsWrite(out,x=lambda[j]* (vector2)T->Point(j)+
		    (1-lambda[j])*(vector2)T->Point((j+1)%3));
	    if (n==0) out<<"mt ";
	    else if (n==1) out<<"lineto ";
	    n++;
	    if (!l) {
	      for (l=T->Vert(j)->Pos(1);l!=T->Vert(j)->end() && (*l)->Dim()==1 && !(*l)->Contains(T->Vert((j+1)%3));l++);	  
	      if (!(*l)->Contains(T->Vert((j+1)%3)))
		Error("internal 38471");
	    }
	  }
	}
	out<<" stroke\n";
	if (l && (*l)->Conta(2)==1) {
	  PsWrite(out,x);
	  out<<"moveto ("<<t<<") sh\n";
	}
      }
    }
    //end level curves...
  }
  out<<"grestore\n";
  out<<"0.5 1 moveto\n";
  out<<"\n\n%%%%%%%%%%% Legenda\n";
  for (int i=0;i<=10;i++) {
    double x=min+(max-min)/10.0*i;
    out<<x<<" sg\n";
    out<<"currentpoint np ";
    out<<"moveto -0.05 0 rlineto 0 0.060 rlineto 0.050 0 rlineto currentpoint\n";
    out<<"closepath fill\n";
    out<<"moveto currentpoint 0 -0.05 rmoveto 0 setgray ("<<x<<") show\n";
    out<<"moveto\n";
  }
}

void MarcaBordi(Cpx *c) {
  Spx *l;
  for (int i=0;i<nV;i++) c->Point(i).bordo=0;
  for (int i=0;i<c->N(1);i++) {
    l=c->Find(1,i);
    if (l->Conta(2)==1) {
      (l->Point(0)).bordo=1;
      (l->Point(1)).bordo=1;
    }
  }
}

void DatoIniziale(Cpx *c) {
  Spx *T;
  Vtx *x;
  vector2 v;
  for (int i=0;i<nV;i++) {
    x=(Vtx *)c->Find(0,i);
    if (x->Point().bordo) {
      int n=0;
      v=vector2(0,0);
      for (Spx::iterator j=x->Pos(1);j!=x->end();j++) {
	if ((*j)->Dim()==1 && (*j)->Conta(2)==1) {
	  T=(*j)->Find(2,0);
	  int k;
	  for (k=0;k<3 && 
		 !((*j)->Contains(T->Vert(k)) 
		   && (*j)->Contains(T->Vert((k+1)%3)));k++);
	  if (k>=3) Error("internal error #93841");
	  v+=versor(T->Point((k+1)%3)-T->Point(k));
	  n++;
	}
      }
      if (n!=2) Error("more than two edges in border curve");
      v/=n;
      x->Point().dato=vector2(v.x[1],-v.x[0]);
      x->Point().dato/=phi(x->Point().dato);
      BordoProj(x->Point());
    }
    else
      x->Point().dato=vector2(0,0);
  }
}

void InitDati(Cpx *c) {
  vector2 v[3];
  Spx *T;
  Spx *x,*y;
  nT=c->N(2); //numero triangoli
  nV=c->N(0); //numero vertici
  int I,J;
  
  M.reserve(nV*nV);
  //  bordo=new vector<int>(nV);
  //  dato=new vector<vector2>(nV);
  divcsi.reserve(nT);

  MarcaBordi(c);  
  DatoIniziale(c);

  for (int i=0;i<3;i++) {
    grads[i].reserve(nT);
    //    vert[i].reserve(nT);
  }
  
  for (int j=0;j<nT;j++) {
    vector2 a,b,w;
    T=c->Find(2,j);
    for (int i=0;i<3;i++) v[i]=T->Point(i);
    for (int i=0;i<3;i++) {
      a=v[(i+1)%3]-v[i];
      b=v[(i+2)%3]-v[i];
      w=-((b*b)*a+(a*a)*b-(a*b)*(a+b))/((a-b)*(a-b));
      ((grads[i]))[j]=w/(w*w);
      //      ((vert[i]))[j]=c->IndexFind(T->Vert(i));
    }
  }
  
  for (int i=0;i<nV;i++) {
    cout<<nV-i<<"  ";cout.flush();
    x=c->Find(0,i);
    for (int j=0;j<nV;j++) {
      y=c->Find(0,j);
      for (int a=0;a<2;a++) (M)[i+j*nV].v[a]=vector2(0,0);
      
      for (Spx::iterator l=x->Pos(2);l!=x->end() && (*l)->Dim()<=2;l++) {
	if ((*l)->Dim()==2 && (*l)->Contains(y)) {
	  int k=c->IndexFind(*l);
	  double A=Area(*l);
	  I=Index(*l,x);
	  J=Index(*l,y);
	  for (int a=0;a<2;a++)
	    for (int b=0;b<2;b++) {
	      (M)[i+j*nV].v[a].x[b]+= A *
		(((grads[I]))[k].x[a] *
		((grads[J]))[k].x[b]);
	    }
	}
      }
    }
  }
  cout<<"\n";
  
  for (int i=0;i<nV;i++) {
    /*    if (c->Point(i).bordo)
      c->Point(i).csi=c->Point(i).dato;*/
    c->Point(i).oldcsi=c->Point(i).csi;
  }

  Energia(c);
  //ora la matrice M e' pronta
}



void Disegna(char *filename,Cpx *c) {
  double e=Energia(c);
  cout<<"Writing picture on Postscript file "<<filename<<"\n";
  ofstream out(filename);
  PsBegin(out);
  
  out <<"newpage";
  
  if (!levelcurves) {
    out<<"\n\n%%%%%%%%%% Divergenza Csi\n";
    PsWritedivcsi(out,c);
  }
  
  out<<"\n\n%%%%%%%%%% Triangolazione\n";
  if (levelcurves)
    out<<"0.8 0.8 0.8 setrgbcolor\n";
  else
    out<<"0 0 0 setrgbcolor\n";
  PsWrite(out,c);
  
  if (levelcurves) {
    out<<"\n\n%%%%%%%%%% Divergenza Csi: curve di livello\n";
    PsWritedivcsi(out,c);
  }
  
  if (csiarrows) {
    out<<"\n\n%%%%%%%%%% Csi\n";
    out<<"1 0 0 setrgbcolor\n";
    PsWritecsi(out,c);
  }
  
  out<<"\n\n%%%%%%%%%% Energia\n";
  out<<"-0.7 -1 moveto 0 setgray\n";
  out<<"(Energy: "<<e<<") show\n"; 
  
  out<<"\n\n%%%%%%%%%% Fine\n";
  out<<"showpage\n";

  PsEnd(out);
}


void Salva(Cpx *c) {
  ofstream out(savefile);
  cerr<<"Writing on file "<<savefile<<"\n";
  out<<Energia(c)<<"\n";
  c->Serialize(out);
  out<<nWulff<<"\n";
}

Cpx *Carica(void) {
  ifstream in(savefile);
  cerr<<"Loading from file "<<savefile<<"\n";
  double e;
  in>>e;
  cerr<<"Energy: "<<e<<"\n";
  Cpx *c=new Cpx(in);
  in>>nWulff;
  InitDati(c);
  return c;
}

double EvolviPasso(Cpx *c,double &dt) {
  point* x;
  for (int i=0;i<nV;i++) {
    x=&(c->Point(i));
    x->oldcsi=x->csi;
    for (int j=0;j<nV;j++) 
      {
	x->csi-= dt*((M)[i+nV*j] * c->Point(j).csi);
	if (!(x->bordo)) {
	  WulffProj(x->csi);
	}
	else 
	  BordoProj(*x);
      }
  }
}

void OldPasso(Cpx *c) {
  for (int i=0;i<nV;i++) {
    c->Point(i).csi=c->Point(i).oldcsi;
  }
}

double EvolviFisso(Cpx *c,double &dt) {
  double e,laste;
  laste=Energia(c);

  while(!interrupt) {
    cout<<"Energy: "<<laste<<" dt: "<<dt<<"              "<<(char)13;
    cout.flush();
    EvolviPasso(c,dt);
    e=Energia(c);
    if (e==laste) return e;
    laste=e;
  }
  ClearInterrupt();
  return e;
}

double Evolvi(Cpx *c,double &dt) {
  int passo=0;
  int n=1;
  double olddt,oldinc;
  vector2 w;
  double laste=10e5,e;

  bool increases=false;

  e=Energia(c);
  while(!interrupt) {
    cout<<"Energy: "<<e<<" dt: "<<dt<<"              "<<(char)13;
    cout.flush();
    laste=e;
    EvolviPasso(c,dt);
    e=Energia(c);
    if (laste<e) {
      if (!increases)
	cout<<"\nEnergy increases!\n";
      increases=true;
      OldPasso(c);
      e=laste;dt/=1.1;
      passo=0;
    }
    if (dt<1e-5) {return e;}

        if (passo%10 == 3) {
      olddt=dt;
      oldinc=laste-e;
      dt/=1.1;      
    }
    if (passo%10 == 4) {
      if (oldinc>laste-e) dt=olddt;
    }
    if (passo%10 == 6 ) {
      olddt=dt;
      oldinc=laste-e;
      dt*=1.1;
    }
    if (passo%10 == 7) {
      if (oldinc>laste-e) dt=olddt;
    }
    passo++;
    n++;
  }
  ClearInterrupt();
  cout<<"\n";
  return e;
}

struct  sigaction Action;
void handler(int a)
{
  if (interrupt)
    {
      cout<<"Interrupt: exiting...\n";
      exit(0);
    }
  cout<<"Ctrl-C signal received...\n";
  interrupt=1;
}

main(int argc,char *argv[]){
  double eps=100.0;
  double dt=0.1;
  double s=0.20;
  int n=4;
  cout<<MYNAME<<" ver. "<<VERSION<<" written by E.Paolini \n";
  void (*fn)(Cpx *,double)=Triangola;
  int cosa=1; /*0: poligono, 1: divano, 2: file*/

  for (int i=1;i<argc;i++) {
    if (!strcmp(argv[i],"eps"))
      eps=atof(argv[++i]);
    else if (!strcmp(argv[i],"n"))
      n=atoi(argv[++i]);
    else if (!strcmp(argv[i],"m"))
      nWulff=atoi(argv[++i]);
    else if (!strcmp(argv[i],"psfile"))
      file=argv[++i];
    else if (!strcmp(argv[i],"file"))
      savefile=argv[++i];
    else if (!strcmp(argv[i],"1"))
      fn=Triangola;
    else if (!strcmp(argv[i],"2"))
      fn=Triangola2;
    else if (!strcmp(argv[i],"dt"))
      dt=atof(argv[++i]);
    else if (!strcmp(argv[i],"s"))
      s=atof(argv[++i]);
    else if (!strcmp(argv[i],"load"))
      cosa=2;
    else if (!strcmp(argv[i],"poligono") || !strcmp(argv[i],"poligon"))
      cosa=0;
    else if (!strcmp(argv[i],"doppiodivano")
	     || !strcmp(argv[i],"doubleseat"))
      cosa=3;
    else if (!strcmp(argv[i],"esagono") || !strcmp(argv[i],"hexagon"))
      cosa=4;
    else if (!strcmp(argv[i],"divano") || !strcmp(argv[i],"seat"))
      cosa=1;
    else if (!strcmp(argv[i],"fitto") || !strcmp(argv[i],"fine"))
      fitt=atof(argv[++i]);
    else if (!strcmp(argv[i],"level"))
      levelcurves=!levelcurves;
    else if (!strcmp(argv[i],"csi"))
      csiarrows=!csiarrows;
    else if (!strcmp(argv[i],"step"))
      levelstep=atof(argv[++i]);
  }

  Action.sa_handler=handler;
  sigaction(SIGINT,&Action,NULL);

  Cpx *c;

  switch(cosa) {
  case 0:
    cout<<"Polygon\n";
    c=poligono(n);
    break;
  case 1:
    cout<<"Seat\n";
    c=divano();
    Aggiusta(c);
    break;
  case 2:
    c=Carica();
    break;
  case 3:
    cout<<"Seat\n";
    c=doppiodivano();
    Aggiusta(c);
    break;
  case 4:
    cout<<"Enloged hexagon\n";
    c=esagono(s,2*nWulff);
    Aggiusta(c);
    break;
  }

  cout<<nWulff*2<<"-agonal wulff shape\n";

  c->Check();
  
  if (cosa!=2) {
    cout<<"Triangulating\n";
    (*fn)(c,eps);
    c->Check();
    cout<<"Computing matrices\n";
    InitDati(c);
  }
  while(1) {
    cout<<"Vertices: "<<c->N(0)<<" Triangles: "<<c->N(2)<<
      " Edges: "<<c->N(1)<<"\n";
    cout<<"Area: "<<Area(c)<<"\n";
    cout<<"Energy: "<<Energia(c)<<" dt: "<<dt<<" eps: "<<eps<<"\n";
    cout<<"a) choose dt\n";
    cout<<"b) evolve with fixed dt\n";
    cout<<"c) evolve\n";
    cout<<"d) draw\n";
    cout<<"e) save\n";
    cout<<"f) triangulate\n";
    cout<<"g) adjust\n";
    cout<<"h) check\n";
    cout<<"x) exit\n";
    char car;
    cout<<"\nYour choice: ";
    cin>>car;
    switch(car) {
    case 'a':
      cout<<"dt=\n";
      cin>>dt;
      break;
    case 'b':
      EvolviFisso(c,dt);
      break;
    case 'c':
      Evolvi(c,dt);
      break;
    case 'd':
      Disegna(file,c);
      break;
    case 'e':
      Salva(c);
      break;
    case 'g':
      Aggiusta(c);
      InitDati(c);
      break;
    case 'f':
      cout<<"eps= ";
      cin>>eps;
      CalcolaDist(c);
      (*fn)(c,eps);
      InitDati(c);
      break;
    case 'h':
      c->Check();
      break;
    case 'x':
      exit(0);
    }
  }
}

