#include "symplex.H"

//extern ostream& operator<<(ostream &,point &);


//fattoriale
int Fact(int n)
{
  int result=1;
  for (;n>1;n--)
    result*=n;
  return result;
}

// n su k
int Parti(int n, int k){
  return Fact(n)/(Fact(k)*Fact(n-k));
}



int Error(char *s)
{
  cerr<<"Error in symplex: "<<s<<"\n";
  exit(1);
};

template<class point>
int Symplex<point>::tab[10][10];

template<class point>
int Symplex<point>::init;

template<class point>
int Symplex<point>::Init(void) {
  cout<<"Inizializza tabella...\n";
  for (int n=0;n<MAXDIM;n++) {
    for (int k=0;k<MAXDIM;k++) {
      if (k==0) tab[n][k]=0;
      else if (k<n)
	tab[n][k]=Parti(n,k)+tab[n][k-1];
      else
	tab[n][k]=tab[n][k-1];
    }
  }
}

template<class point>
int Symplex<point>::Contains(const Symplex *const p)
{
  register iterator i;
  for (i=Pos(p->Dim());i!=end() && (*i)!=p;i++);
  return i!=end();
}

template<class point>
int Symplex<point>::Conta(int dim) {
  iterator i,j;
  for (i=Pos(dim);i!=end() && (*i)->Dim()<dim;i++);
  for (j=i;j!=end() && (*j)->Dim()==dim;j++);
  return j-i;
}

template<class point>
int Symplex<point>::AddNew(Symplex *p){
  iterator i;
  for(i=Pos(p->Dim());i!=end() && (*i)->Dim()<p->Dim();i++);
  insert(i,p);
}

template<class point>
Symplex<point> *Complex<point>::AddSymplex(Symplex<point>*p){
  insert(begin()+tab[p->Dim()+1],p);
  for (int i=p->Dim()+1;i<MAXDIM;i++)
    tab[i]++;
  return p;
}

template<class point>
void Complex<point>::subsetadd(vector<Symplex<point>*> &res,vector<Vertex<point> *> &p,int k) {
  //aggiunge a res il sottosimplesso p[0]...p[k-1], creandolo se necessario
  if (k==1) res.push_back((Symplex<point>*)(p[0]));
  else {
    for (Symplex<point>::iterator i=p[0]->Pos(k-1);i!=p[0]->end();i++) {
      if ((*i)->Dim()==k-1) {
	int j;
	for (j=1;j<k && (*i)->Contains(p[j]);j++);
	if (j==k) {// ok, l'ho trovato 
	  res.push_back(*i);
	  return;
	}
      }
    }
    // il simplesso sembra non esistere!!, lo creo!
    res.push_back(NewSymplex(p,k));
  }
}

template<class point>
void Complex<point>::getsubsets(vector<Symplex<point>*> &res,vector<Vertex<point> *> &p,int a,int b,int k) {
  Vertex<point> *swap;
  // aggiunge a res tutti i sottosimplessi (creandoli se necessario) di p 
  // composti da p[0]...p[a-1] e altri k elementi scelti tra p[a]...p[b-1]
  for (int i=0;i<b-a-k+1;i++) {
    if (k==1)
      subsetadd(res,p,a+1);
    else
      getsubsets(res,p,a+1,b-i,k-1);
    swap=p[a];
    p.erase(p.begin()+a);
    p.insert(p.begin()+b-1,swap);
  }
}

template<class point>
Symplex<point>* Complex<point>::NewSymplex(vector<Vertex<point> *> &p,const int n) {
  // il numero di sottosimplessi e' 2^n-2 
  // (devo togliere il vuoto e se stesso
  vector<Symplex<point>*> list;
  list.reserve((1<<n)-2);
  
  for (int k=1;k<n;k++) {
    //aggiunge tutti i k-sottoinsiemi
    getsubsets(list,p,0,n,k);
  }
  return AddSymplex(new Symplex<point>(list,n-1));
}

template<class point>
int Complex<point>::IndexFind(Symplex<point>*s) {
  int i;
  for (i=tab[s->Dim()];*(begin()+i)!=s;i++);
#ifdef DEBUG
  if (s!=Find(s->Dim(),i-tab[s->Dim()]))
    Error("IndexFind non funziona!");
#endif
  return i-tab[s->Dim()];
}

template<class point>
Symplex<point>::Symplex(vector<Symplex<point>*> &p,const int d):vector<Symplex<point>*>(p), dim(d)
{  
  for (iterator i=begin();i!=end();i++)
    (*i)->Add(this);
#ifdef DEBUG
  Check();
#endif
}

template<class point>
void Symplex<point>::Del(Symplex<point>*p)
{
  iterator i=begin();
  for (i=Pos(p->Dim());i!=end() && (*i)!=p;i++);
  if ((*i)==p) erase(i);
}

template<class point>
int Symplex<point>::Check() { //controlla autoconsistenza del simplesso
  vector<int> dim(MaxDim()+1);
  int max=-1;

  if (size()==0 && Dim()==0) return 1;
  for (int i=MaxDim();i>=0;i--) dim[i]=0;

  for (iterator i=begin();(i+1)!=end();i++){
    //controlla che i sottosimplessi siano in ordine di dimensione
    if ((*i)->Dim()>(*(i+1))->Dim()) {
      FullWrite(cerr);
      Error("I sottosimplessi non sono ordinati per dimensione");
    }
    if ((i-begin())<tab[Dim()][(*i)->Dim()]) {
      FullWrite(cerr);
      Error("C'e` un errore in tab");
    }
  }
    
  for (iterator i=begin();i!=end();i++){
    dim[(*i)->Dim()]++;
  }
  for (int i=0;i<Dim();i++){
    if (dim[i]!=Parti(Dim()+1,i+1)){
      FullWrite(cerr);
      Error("Il simplesso non ha abbastanza facce"); //il numero di sottosimplessi non e' giusto
      return 0;
    }
  }
  if (dim[Dim()]!=0) {
    FullWrite(cerr);
    Error("Il simplesso si autocontiene"); //c'e' un sottosimplesso con la mia dimensione
    return 0;
  }

  // controlla che non ci siano duplicati
  for (iterator i=begin();i!=end();i++)
    for (iterator j=i;j!=end();j++)
      if (j!=i && (*i)==(*j)) 
	{
	  FullWrite(cerr);
	  Error("Il simplesso ha facce duplicate"); //sottosimplesso doppio 
	}

  // controlla che ogni sottosiplesso mi contenga
  for (iterator i=begin();i!=end();i++)
    if (!(*i)->Contains(this))
      {
	FullWrite(cerr);
	Error("Un sottosimplesso non mi reciproca");
      }
  return 1;

  if (Dim()>0) {
    // controlla che non ci sia un simplesso uguale a me
    int count=0;
    for (Symplex<point>::iterator i=(*begin())->begin();i!=(*begin())->end();i++)
      if ((*i)==this) count++;
    if (count>1) {
      FullWrite(cerr);
      Error("C'e' un simplesso duplicato");
    }
  }
}

template<class point>
int Complex<point>::Check(){
  for (iterator i=begin();i!=end();i++) {
    if (!( i-begin() >=tab[(*i)->Dim()] && i-begin()<tab[(*i)->Dim()+1]))
      Error("I simplessi non sono ben ordinati nel complesso");
    if (*i==0) Error("Null pointer in Check");
    else if (!((*i)->Check())) return 0;
  }
  return 1;
}
       
template<class point>
void Symplex<point>::Write(ostream &out=cout){
  out<<"{";
  if (Dim()>0)
    for (int i=0;i<Dim()+1;i++) out<<Point(i);
  else
    out<<((Vertex<point> *)this)->Point();
  out<<"|"<<size()<<"}";
}

template<class point>
void Symplex<point>::FullWrite(ostream &out=cout){
  Write(out);
  out<<"=[";
  for (iterator i=begin();i!=end();i++) {
    (*i)->Write(out);
    out<<",";
  }
  out<<"]\n";
}

template<class point>
void Complex<point>::Write(ostream &out=cout){
  out<<"Complesso: \n";
  for (iterator i=begin();i!=end();i++)
    {
      (*i)->Write(out);
      out<<"\n";
    }
}

template<class point>
Symplex<point>* Symplex<point>::Find(int dim,int k=0)
{
  for (iterator i=Pos(dim);i!=end();i++)
    if ((*i)->Dim()==dim) {
      if (k==0) return (*i);
      k--;
    }
  return 0;
}

template<class point>
int Symplex<point>::MaxDim(void) {
  int k=Dim();
  for (iterator i=begin();i!=end();i++)
    if ((*i)->Dim()>k) k=((*i)->Dim());
  return k;
}

template<class point>
Symplex<point>::~Symplex()
{
#ifdef DEBUG
  Check();
#endif
  for (iterator i=begin();i!=end();i++)
    {
      if ((*i)->Dim()>=Dim()) Error("Non puoi cancellare un simplesso non massimale");
      else (*i)->Del(this);
    }
}

template<class point>
void Complex<point>::Delete(Symplex<point>*s)
{
  Symplex<point>*t;
  iterator i;
  for (i=begin()+tab[s->Dim()];
#ifdef DEBUG
       i!=end() && 
#endif
	 (*i)!=s;i++);
#ifdef DEBUG
  if (i==end()) Error("Delete non existent symplex!");
#endif
  erase(i);
  for (int j=s->Dim()+1;j<MAXDIM;j++) tab[j]--;
  delete s; 
}

template<class point>
void Complex<point>::Serialize(ostream &out) {
  out<<size()<<"\n";
  for (int dim=0;N(dim)>0;dim++) {
    for (int i=0;i<N(dim);i++) {
      out<<dim<<" ";
      if (dim>0)
	for (int k=0;k<dim+1;k++) {
	  out<<IndexFind(Find(dim,i)->Vert(k))<<" ";
	}
      else
	{
	  out<<Point(i);
	}
      out<<"\n";
    }
  }
}

template<class point>
Complex<point>::Complex(istream &in) {
  int n,dim;
  for (int i=0;i<MAXDIM;i++) tab[i]=0;
  in>>n;
  for (int i=0;i<n;i++) {
    in>>dim;
    if (dim==0) {
      Symplex<point> *s;
      point v;
      in>>v;
      s=NewSymplex(v);
      if (Find(0,i)!=s) Error("Errore interno 3941");
    }
    else {
      vector<Vertex<point> *> v(dim+1);
      int n;
      for (int k=0;k<dim+1;k++) {
	in>>n;
	v[k]=(Vertex<point> *)Find(0,n);
      }
      NewSymplex(v);
    }
  }
}

//aggiunge *v in mezzo al simplesso
template<class point>
void Complex<point>::Triangola(Symplex<point> *s,Vertex<point> *v) {
  int k,d,mxdm;
  Symplex<point> *t;
  static vector<Vertex<point> *> x,y;
  static vector<Symplex<point> *> delenda(0);
  Vertex<point> *v0,*v1;

#ifdef DEBUG
  Check();
#endif


  if (s->Dim()==1) {
    mxdm=s->MaxDim();

    x.reserve(mxdm+1);  // perche' non esiste resize???
    y.reserve(mxdm+1);  //
    for (int i=x.size();i<mxdm+1;i++) {
      x.push_back(v0);  //ci potevo mettere qualunque cosa...
      y.push_back(v0);  //
    }
#ifdef DEBUG
    if (x.size()<mxdm+1 || y.size()<mxdm+1) Error("interno 328437");
    if (delenda.size()!=0) Error("interno 34214");
#endif

    v0=s->Vert(0);
    v1=s->Vert(1);
    x[0]=v0;
    y[1]=v1;
    x[1]=y[0]=v;
    NewSymplex(x,2);
    NewSymplex(y,2);
    delenda.push_back(s);

#ifdef DEBUG
  Check();
#endif


    for (d=s->Dim()+1;d<=mxdm;d++)
      for(int l=0;t=s->Find(d,l);l++)
	{
	  //t e' un soprasimplesso di s 
	  for (k=0;k<t->Dim()+1;k++) {
	    x[k]=y[k]=t->Vert(k);
	    if (x[k]==v0) x[k]=v;
	    else if (y[k]==v1) y[k]=v;
	  }
	  NewSymplex(x,d+1);
	  NewSymplex(y,d+1);
	  delenda.push_back(t);
	}
    
    //ora devo cancellare i simplessi in ordine inverso da delenda
    //Write();
    while (delenda.begin()!=delenda.end()) {
      Delete(delenda.back());
      delenda.pop_back();
    }
    //Write();
  }
  else
    Error("So triangolare solo segmenti");
#ifdef DEBUG
  Check();
#endif
}


