#ifndef __SYMPLEX_H__
#define __SYMPLEX_H__

#include <vector>
#include <iostream.h>
#define MAXDIM 10

extern int Error(char *);

template <class point> class Vertex;
template <class point> class Symplex;
template <class point> class Complex;

template <class point> 
class Symplex:public vector<Symplex<point> *>{
  // contiene i sottosimplessi e soprasimplessi in ordine di dimensione
public:
protected:
  static int tab[MAXDIM][MAXDIM]; //tabella quadrata
                          //tab[n][k]=Sum_{j<k} Parti(n,j+1)
  static int init;
  static int Init(void);

  int dim; //dimensione del simplesso
  int AddNew(Symplex *p);
  int Add(Symplex *p){if (!Contains(p)) AddNew(p);}
  void Del(Symplex *p);
  Symplex():dim(-1){};
public:
  int Spanned(vector<Vertex<point> *> &p);
  int Dim() const {return dim;}
  Symplex **Pos(int d) {return begin()+tab[Dim()][d];}
  int Check();
  void Write(ostream &out=cout);
  void FullWrite(ostream &out=cout);
  int Contains(const Symplex *const);
  Symplex(vector<Symplex *> &p,const int dim);
  ~Symplex();
  inline point &Point(int n); // torna l'ennesimo punto di vertice
  inline Vertex<point> *Vert(int n); // torna l'ennesimo vertice
  Symplex* Find(int dim,int k); //torna il k-esimo dim-simplesso 
  int MaxDim(); //torna la dimensione massima dei simplessi contenenti this
  int Conta(int dim); //torna il numero di symp di dimensione dim
  friend class Complex<point>;
  friend class SympIterator;
};

template <class point>
class Vertex:public Symplex<point>{
  point v; //coordinate vertice
public:
  point &Point(){return v;}
  Vertex(const point &p):v(p){dim=0;}
};

template <class point>
class Complex:public vector<Symplex<point> *>
{
protected:
  int tab[MAXDIM]; //primo simplesso di ogni dimensione
  Symplex<point> *AddSymplex(Symplex<point> *p);
  void subsetadd(vector<Symplex<point> *> &res,
		 vector<Vertex<point> *> &p,int k);
  void getsubsets(vector<Symplex<point> *> &res,vector<Vertex<point> *>&p,
		  int a,int b,int k);
public:
  void Write(ostream &out=cout);
  int Check(void);
  
  Complex(){for (int i=0;i<MAXDIM;i++) tab[i]=0;}
  Complex(istream &in);

  //aggiunge un simplesso, dati i suoi vertici, se alcuni sottosimplessi non

  //esistono, vengono creati.
  Symplex<point>* NewSymplex(vector<Vertex<point> *> &p,const int size);
  
  Symplex<point>* NewSymplex(vector<Vertex<point> *> &p) {return NewSymplex(p,p.size());}

  //aggiunge un simplesso, dati tutti i suoi sottosimplessi
  Symplex<point>* NewSymplex(vector<Symplex<point> *> &list, int dim) {
    return AddSymplex(new Symplex<point>(list,dim));
  }
  
  //aggiunge un vertice sconnesso:
  Vertex<point>* NewSymplex(const point &p){
    return (Vertex<point> *)AddSymplex(new Vertex<point>(p));
  }
  inline int N(); //numero di simplessi
  inline int N(int n); //numero di simplessi di dim n
  void Triangola(Symplex<point> *s,Vertex<point> *v);
  void Delete(Symplex<point> *s);
  inline 
    Symplex<point> *Find(int dim,int n);/*torna l'n-esimo simplesso dim-dimensionale*/
  Vertex<point> *Vert(int n){return (Vertex<point> *)Find(0,n);}
  point &Point(int n){return Vert(n)->Point();};
  Symplex<point> **Begin(int dim){return begin()+tab[dim];};
  Symplex<point> **End(int dim){return begin()+tab[dim+1];};
  int IndexFind(Symplex<point> *s); /* torna l'indice n t.c. s=Find(s->Dim(),n) */
  void Serialize(ostream &out);
};

template<class point>
Vertex<point>* Symplex<point>::Vert(int n)
{
#ifdef DEBUG
  if (n>size())
    Error("Point out of bounds");
  if ((*(begin()+n))->Dim()!=0)
    Error("Il punto non e' un punto");
#endif
  return ((Vertex<point> *)*(begin()+n));
}

template<class point>
point& Symplex<point>::Point(int n)
{
  return Vert(n)->Point();
}

template<class point>
int Complex<point>::N(void) {
  return size();
}

template<class point>
int Complex<point>::N(int dim){
  return tab[dim+1]-tab[dim];
}

template<class point>
Symplex<point> *Complex<point>::Find(int dim,int n) {
#ifdef DEBUG
  if (n>=N(dim)) Error("Find non trova!");
#endif
  return *(begin()+tab[dim]+n);
}

#endif

